@@ -1,4 +1,4 @@
-/* Route object related header for route server.
+/* BGP routing information base
  * Copyright (C) 1996, 97, 98, 2000 Kunihiro Ishiguro
  *
  * This file is part of GNU Zebra.
@@ -22,26 +22,23 @@
 #ifndef _ZEBRA_BGP_ROUTE_H
 #define _ZEBRA_BGP_ROUTE_H
 
-/* I want to change structure name from bgp_route to bgp_info. */
 struct bgp_info
 {
   /* For linked list. */
   struct bgp_info *next;
   struct bgp_info *prev;
 
-  /* Type of this prefix */
+  /* BGP route type.  This can be static, RIP, OSPF, BGP etc.  */
   u_char type;
 
-  /* Type of bgp prefix. */
+  /* When above type is BGP.  This sub type specify BGP sub type
+     information.  */
+  u_char sub_type;
 #define BGP_ROUTE_NORMAL    0
 #define BGP_ROUTE_STATIC    1
 #define BGP_ROUTE_AGGREGATE 2
-  u_char sub_type;
 
-  /* Selected route flag. */
-  u_char as_selected;
-
-  /* BGP info status. */
+  /* BGP information status.  */
   u_char flags;
 #define BGP_INFO_IGP_CHANGED    (1 << 0)
 #define BGP_INFO_DAMPED         (1 << 1)
@@ -49,49 +46,81 @@ struct bgp_info
 #define BGP_INFO_SELECTED       (1 << 3)
 #define BGP_INFO_VALID          (1 << 4)
 #define BGP_INFO_ATTR_CHANGED   (1 << 5)
+#define BGP_INFO_DMED_CHECK     (1 << 6)
+#define BGP_INFO_DMED_SELECTED  (1 << 7)
 
-  /* Pointer to peer structure. */
+  /* Peer structure.  */
   struct peer *peer;
 
-  /* Pointer to attributes structure. */
+  /* Attribute structure.  */
   struct attr *attr;
 
-  /* Aggregate related information. */
+  /* This route is suppressed with aggregation.  */
   int suppress;
   
-  /* Nexthop reachability check. */
+  /* Nexthop reachability check.  */
   u_int32_t igpmetric;
 
-  /* Time */
+  /* Uptime.  */
   time_t uptime;
 
-  /* Pointer to dampening structure */
+  /* Pointer to dampening structure.  */
   struct bgp_damp_info *bgp_damp_info;
 
-  /* Tag */
+  /* MPLS label.  */
   u_char tag[3];
 };
 
 /* BGP static route configuration. */
 struct bgp_static
 {
-  safi_t safi;
+  /* Backdoor configuration.  */
   int backdoor;
+
+  /* Import check status.  */
   u_char valid;
+
+  /* IGP metric. */
   u_int32_t igpmetric;
+
+  /* MPLS label.  */
+  u_char tag[3];
 };
 
+#define DISTRIBUTE_IN_NAME(F)   ((F)->dlist[FILTER_IN].name)
+#define DISTRIBUTE_IN(F)        ((F)->dlist[FILTER_IN].alist)
+#define DISTRIBUTE_OUT_NAME(F)  ((F)->dlist[FILTER_OUT].name)
+#define DISTRIBUTE_OUT(F)       ((F)->dlist[FILTER_OUT].alist)
+
+#define PREFIX_LIST_IN_NAME(F)  ((F)->plist[FILTER_IN].name)
+#define PREFIX_LIST_IN(F)       ((F)->plist[FILTER_IN].plist)
+#define PREFIX_LIST_OUT_NAME(F) ((F)->plist[FILTER_OUT].name)
+#define PREFIX_LIST_OUT(F)      ((F)->plist[FILTER_OUT].plist)
+
+#define FILTER_LIST_IN_NAME(F)  ((F)->aslist[FILTER_IN].name)
+#define FILTER_LIST_IN(F)       ((F)->aslist[FILTER_IN].aslist)
+#define FILTER_LIST_OUT_NAME(F) ((F)->aslist[FILTER_OUT].name)
+#define FILTER_LIST_OUT(F)      ((F)->aslist[FILTER_OUT].aslist)
+
+#define ROUTE_MAP_IN_NAME(F)    ((F)->map[FILTER_IN].name)
+#define ROUTE_MAP_IN(F)         ((F)->map[FILTER_IN].map)
+#define ROUTE_MAP_OUT_NAME(F)   ((F)->map[FILTER_OUT].name)
+#define ROUTE_MAP_OUT(F)        ((F)->map[FILTER_OUT].map)
+
 /* Prototypes. */
 void bgp_route_init ();
 void bgp_announce_table (struct peer *);
 void bgp_refresh_table (struct peer *, afi_t, safi_t);
 void bgp_route_clear (struct peer *);
 void bgp_soft_reconfig_in (struct peer *, afi_t, safi_t);
+void bgp_adj_clear (struct route_table *, safi_t);
 
 int nlri_sanity_check (struct peer *, int, u_char *, bgp_size_t);
 int nlri_parse (struct peer *, struct attr *, struct bgp_nlri *);
 
-void bgp_redistribute_add (struct prefix *, struct in_addr *, u_char);
+int bgp_maximum_prefix_overflow (struct peer_conf *, afi_t, safi_t);
+
+void bgp_redistribute_add (struct prefix *, struct in_addr *, u_int32_t, u_char);
 void bgp_redistribute_delete (struct prefix *, u_char);
 void bgp_redistribute_withdraw (struct bgp *, afi_t, int);
 
@@ -100,10 +129,11 @@ int bgp_static_set_vpnv4 (struct vty *vty, char *, char *, char *);
 
 int bgp_static_unset_vpnv4 (struct vty *, char *, char *, char *);
 
-int bgp_config_write_network (struct vty *, struct bgp *, afi_t);
+int bgp_config_write_network (struct vty *, struct bgp *, afi_t, safi_t, int *);
 int bgp_config_write_distance (struct vty *, struct bgp *);
 
-void route_vty_out_detail (struct vty *, struct prefix *, struct bgp_info *);
+void route_vty_out_detail (struct vty *, struct prefix *, struct bgp_info *,
+			   afi_t, safi_t);
 
 void bgp_aggregate_increment (struct bgp *, struct prefix *, struct bgp_info *,
 			      afi_t, safi_t);
@@ -112,4 +142,7 @@ void bgp_aggregate_decrement (struct bgp *, struct prefix *, struct bgp_info *,
 
 u_char bgp_distance_apply (struct prefix *, struct bgp_info *, struct bgp *);
 
+afi_t bgp_node_afi (struct vty *);
+safi_t bgp_node_safi (struct vty *);
+
 #endif /* _ZEBRA_BGP_ROUTE_H */
