@@ -32,7 +32,7 @@ buffer_data_new (size_t size)
   struct buffer_data *d;
 
   d = XMALLOC (MTYPE_BUFFER_DATA, sizeof (struct buffer_data));
-  bzero (d, sizeof (struct buffer_data));
+  memset (d, 0, sizeof (struct buffer_data));
   d->data = XMALLOC (MTYPE_BUFFER_DATA, size);
 
   return d;
@@ -48,14 +48,13 @@ buffer_data_free (struct buffer_data *d)
 
 /* Make new buffer. */
 struct buffer *
-buffer_new (int type, size_t size)
+buffer_new (size_t size)
 {
   struct buffer *b;
 
   b = XMALLOC (MTYPE_BUFFER, sizeof (struct buffer));
-  bzero (b, sizeof (struct buffer));
+  memset (b, 0, sizeof (struct buffer));
 
-  b->type = type;
   b->size = size;
 
   return b;
@@ -398,6 +397,13 @@ buffer_flush_vty (struct buffer *b, int fd, int size,
   struct buffer_data *out;
   struct buffer_data *next;
 
+#ifdef  IOV_MAX
+  int iov_size;
+  int total_size;
+  struct iovec *c_iov;
+  int c_nbytes;
+#endif /* IOV_MAX */
+
   /* For erase and more data add two to b's buffer_data count.*/
   if (b->alloc == 1)
     iov = small_iov;
@@ -445,7 +451,42 @@ buffer_flush_vty (struct buffer *b, int fd, int size,
     }
 
   /* We use write or writev*/
-  nbytes = writev (fd, iov, iov_index);
+
+#ifdef IOV_MAX
+  /* IOV_MAX are normally defined in <sys/uio.h> , Posix.1g.
+     example: Solaris2.6 are defined IOV_MAX size at 16.     */
+  c_iov = iov;
+  total_size = iov_index;
+  nbytes = 0;
+
+  while( total_size > 0 )
+    {
+       /* initialize write vector size at once */
+       iov_size = ( total_size > IOV_MAX ) ? IOV_MAX : total_size;
+
+       c_nbytes = writev (fd, c_iov, iov_size );
+
+       if( c_nbytes < 0 )
+         {
+           if(errno == EINTR)
+             ;
+             ;
+           if(errno == EWOULDBLOCK)
+             ;
+             ;
+           nbytes = c_nbytes;
+           break;
+
+         }
+
+        nbytes += c_nbytes;
+
+       /* move pointer io-vector */
+       c_iov += iov_size;
+       total_size -= iov_size;
+    }
+#else  /* IOV_MAX */
+   nbytes = writev (fd, iov, iov_index);
 
   /* Error treatment. */
   if (nbytes < 0)
@@ -455,6 +496,7 @@ buffer_flush_vty (struct buffer *b, int fd, int size,
       if (errno == EWOULDBLOCK)
 	;
     }
+#endif /* IOV_MAX */
 
   /* Free printed buffer data. */
   for (out = b->head; out && out != data; out = next)
@@ -486,7 +528,6 @@ buffer_flush_window (struct buffer *b, int fd, int width, int height,
   unsigned long size;
   int lp;
   int lineno;
-  int ret;
   struct buffer_data *data;
 
   if (height >= 2)
@@ -523,57 +564,5 @@ buffer_flush_window (struct buffer *b, int fd, int width, int height,
   /* Write data to the file descriptor. */
  flush:
 
-#ifdef DEBUG
-  printf ("cp:%ld lp:%d size:%ld ineno:%d\n",
-	  cp, lp, size, lineno);
-#endif /* DEBUG */
-
-  ret = buffer_flush_vty (b, fd, size, erase, no_more);
-
-  return ret;
+  return buffer_flush_vty (b, fd, size, erase, no_more);
 }
-
-void
-buffer_dump (struct buffer *b)
-{
-  struct buffer_data *d;
-
-  printf ("buffer type %d size %ld alloc %ld\n", b->type, b->size, b->alloc);
-
-  for (d = b->head; d; d = d->next)
-    printf ("  data cp:%ld sp:%ld\n", d->cp, d->sp);
-}
-
-#ifdef TEST
-main ()
-{
-  struct buffer *b;
-  char kuni[] = "kunihiro\n";
-  char mio[] = "miomiomi\n";
-
-  b = buffer_new (BUFFER_VTY, 3);
-  buffer_write (b, kuni, sizeof kuni);
-  buffer_write (b, mio, sizeof mio);
-
-  buffer_dump (b);
-
-  buffer_flush_vty (b, 0, 20, 0);
-  buffer_flush_vty (b, 0, 5, 0);
-  /* buffer_dump (b); */
-  buffer_flush_vty (b, 0, 5, 0);
-  /* buffer_dump (b); */
-  printf ("\n");
-
-  exit (0);
-
-  printf ("cleared\n");
-  buffer_reset (b);
-  buffer_dump (b);
-
-  buffer_write (b, kuni, sizeof kuni);
-  buffer_write (b, kuni, sizeof kuni);
-
-  printf ("\n");
-  buffer_flush_all (b, 0);
-}
-#endif /* TEST */
