@@ -25,42 +25,22 @@
 
 #include "linklist.h"
 
-/* Interface name length.
+/*
+  Interface name length.
 
- Linux 2.0.36 and Linux 2.1.131 define this value in
- /usr/include/linux/if.h and it's like this:
-
- #define    IFNAMSIZ        16
-
- FreeBSD 2.2.7 and 3.0 define this value in
- /usr/include/net/if.h and it's like this:
+   Linux define value in /usr/include/linux/if.h.
+   #define IFNAMSIZ        16
 
+   FreeBSD define value in /usr/include/net/if.h.
+   #define IFNAMSIZ        16
 */
 
 #define INTERFACE_NAMSIZ      20
 #define INTERFACE_HWADDR_MAX  20
 
-#ifdef HAVE_IF_PSEUDO
-#define IF_PSEUDO      0x01
-#define IF_PSEUDO_SET(IF) (((IF)->status) |= IF_PSEUDO)
-#define IF_PSEUDO_UNSET(IF) (((IF)->status) &= ~IF_PSEUDO)
-#define IS_IF_PSEUDO(IF) (((IF)->status) & IF_PSEUDO)
-
-#if 0
-#define IF_UNKOWN      0x02
-#define IF_UNKNOWN_SET(IF) (((IF)->status) |= IF_UNKNOWN)
-#define IF_UNKNOWN_UNSET(IF) (((IF)->status) &= ~IF_UNKNOWN)
-#define IS_IF_UNKNOWN(IF) (((IF)->status) & IF_UNKNOWN)
-#endif
-#endif /* HAVE_IF_PSEUDO */
-
-#ifndef INTERFACE_UNKNOWN
-#define INTERFACE_UNKNOWN 1000000
-#endif /* INTERFACE_UNKNOWN */
-
-#ifndef INTERFACE_PSEUDO
-#define INTERFACE_PSEUDO 0
-#endif  /* INTERFACE_PSEUDO */
+/* Internal If indexes start at 0xFFFFFFFF and go down to 1 greater
+   than this */
+#define IFINDEX_INTERNBASE 0x80000000
 
 #ifdef HAVE_PROC_NET_DEV
 struct if_stats
@@ -105,6 +85,8 @@ struct interface
 
   /* Zebra internal interface status */
   u_char status;
+#define ZEBRA_INTERFACE_ACTIVE     (1 << 0)
+#define ZEBRA_INTERFACE_SUB        (1 << 1)
   
   /* Interface flags. */
   unsigned long flags;
@@ -112,7 +94,7 @@ struct interface
   /* Interface metric */
   int metric;
 
-  /* INterface MTU. */
+  /* Interface MTU. */
   int mtu;
 
   /* Hardware address. */
@@ -136,10 +118,6 @@ struct interface
 
   /* Connected address list. */
   list connected;
-#if 0
-  struct list *ifa_v4;
-  struct list *ifa_v6;
-#endif /* 0 */
 
   /* Daemon specific interface data pointer. */
   void *info;
@@ -159,9 +137,21 @@ struct connected
   /* Attached interface. */
   struct interface *ifp;
 
+  /* Flags for configuration. */
+  u_char conf;
+#define ZEBRA_IFC_REAL         (1 << 0)
+#define ZEBRA_IFC_CONFIGURED   (1 << 1)
+
+  /* Flags for connected address. */
+  u_char flags;
+#define ZEBRA_IFA_SECONDARY   (1 << 0)
+
   /* Address of connected network. */
   struct prefix *address;
   struct prefix *destination;
+
+  /* Label for Linux 2.2.X and upper. */
+  char *label;
 };
 
 /* Interface hook sort. */
@@ -193,7 +183,7 @@ struct connected
 /* Prototypes. */
 struct interface *if_new (void);
 struct interface *if_create (void);
-struct interface *if_lookup_by_index (int);
+struct interface *if_lookup_by_index (unsigned int);
 struct interface *if_lookup_by_name (char *);
 struct interface *if_lookup_exact_address (struct in_addr);
 struct interface *if_lookup_address (struct in_addr);
