@@ -38,7 +38,7 @@ struct list
   struct listnode *tail;
   unsigned int count;
   int (*cmp) (void *val1, void *val2);
-  int (*del) (void *val);
+  void (*del) (void *val);
 };
 
 #define nextnode(X) ((X) = (X)->next)
@@ -47,28 +47,54 @@ struct list
 #define list_isempty(X) ((X)->head == NULL && (X)->tail == NULL)
 #define getdata(X) ((X)->data)
 
-list list_init();
-listnode list_lookup_node (list, void *);
-void list_add_node (list, void *);
-void list_add_sort_node (list, void *);
+/* Prototypes. */
+struct list *list_new();
+void list_free (struct list *);
+
+void listnode_add (struct list *, void *);
+void listnode_add_sort (struct list *, void *);
+void listnode_add_after (struct list *, struct listnode *, void *);
+void listnode_delete (struct list *, void *);
+struct listnode *listnode_lookup (struct list *, void *);
+
+void list_delete (struct list *);
+void list_delete_all_node (struct list *);
+
+/* For ospfd and ospf6d. */
+void list_delete_node (list, listnode);
+
+/* For ospf_spf.c */
 void list_add_node_prev (list, listnode, void *);
 void list_add_node_next (list, listnode, void *);
 void list_add_list (list, list);
-void list_delete_by_val (list, void *);
-void list_delete_all_node (list);
-void list_delete_all (list);
-void list_free (list);
-void list_delete_node (list, listnode);
-
-struct list *list_new ();
-
-/* From newlist.c */
-void list_delete (struct list *);
-void *listnode_delete (struct list *list, void *val);
 
 /* List iteration macro. */
 #define LIST_LOOP(L,V,N) \
   for ((N) = (L)->head; (N); (N) = (N)->next) \
     if (((V) = (N)->data) != NULL)
 
+/* List node add macro.  */
+#define LISTNODE_ADD(L,N) \
+  do { \
+    (N)->prev = (L)->tail; \
+    if ((L)->head == NULL) \
+      (L)->head = (N); \
+    else \
+      (L)->tail->next = (N); \
+    (L)->tail = (N); \
+  } while (0)
+
+/* List node delete macro.  */
+#define LISTNODE_DELETE(L,N) \
+  do { \
+    if ((N)->prev) \
+      (N)->prev->next = (N)->next; \
+    else \
+      (L)->head = (N)->next; \
+    if ((N)->next) \
+      (N)->next->prev = (N)->prev; \
+    else \
+      (L)->tail = (N)->prev; \
+  } while (0)
+
 #endif /* _ZEBRA_LINKLIST_H */
