@@ -29,8 +29,6 @@
 enum
 {
   MTYPE_TMP = 1,
-  MTYPE_COMMAND,
-  MTYPE_COMMAND_CONST,
   MTYPE_STRVEC,
   MTYPE_VECTOR,
   MTYPE_VECTOR_INDEX,
@@ -47,7 +45,6 @@ enum
   MTYPE_AS_PATH,
   MTYPE_COMMUNITY,
   MTYPE_COMMUNITY_VAL,
-  MTYPE_COMMUNITY_REGEXP,
   MTYPE_ECOMMUNITY,
   MTYPE_ECOMMUNITY_VAL,
   MTYPE_CLUSTER,
@@ -62,8 +59,6 @@ enum
   MTYPE_STREAM_DATA,
   MTYPE_STREAM_FIFO,
   MTYPE_BGP_PEER,
-  MTYPE_RADIX_NODE,
-  MTYPE_RADIX_MASK,
   MTYPE_PREFIX,
   MTYPE_PREFIX_IPV4,
   MTYPE_PREFIX_IPV6,
@@ -73,7 +68,6 @@ enum
   MTYPE_RIPNG_AGGREGATE,
   MTYPE_ROUTE_TABLE,
   MTYPE_ROUTE_NODE,
-  MTYPE_RIPNG_SLOT,
   MTYPE_ACCESS_LIST,
   MTYPE_ACCESS_LIST_STR,
   MTYPE_ACCESS_FILTER,
@@ -98,8 +92,6 @@ enum
   MTYPE_OSPF_IF,
   MTYPE_OSPF_NEIGHBOR,
   MTYPE_OSPF_ROUTE,
-  MTYPE_OSPF_ADDR,
-  MTYPE_OSPF_MESSAGE,
   MTYPE_OSPF_TMP,
   MTYPE_OSPF_LSA,
   MTYPE_OSPF_LSA_DATA,
@@ -112,6 +104,7 @@ enum
   MTYPE_OSPF_VL_DATA,
   MTYPE_OSPF_CRYPT_KEY,
   MTYPE_OSPF_EXTERNAL_INFO,
+  MTYPE_OSPF_MESSAGE,
   MTYPE_OSPF6_TOP,
   MTYPE_OSPF6_AREA,
   MTYPE_OSPF6_IF,
@@ -120,6 +113,8 @@ enum
   MTYPE_OSPF6_PREFIX,
   MTYPE_OSPF6_MESSAGE,
   MTYPE_OSPF6_LSA,
+  MTYPE_OSPF6_LSA_SUMMARY,
+  MTYPE_OSPF6_LSDB,
   MTYPE_OSPF6_VERTEX,
   MTYPE_OSPF6_SPFTREE,
   MTYPE_OSPF6_NEXTHOP,
@@ -132,14 +127,12 @@ enum
   MTYPE_AS_LIST,
   MTYPE_COMMUNITY_ENTRY,
   MTYPE_COMMUNITY_LIST,
+  MTYPE_COMMUNITY_REGEXP,
   MTYPE_ZCLIENT,
   MTYPE_NEXTHOP,
   MTYPE_RTADV_PREFIX,
   MTYPE_IF_RMAP,
-  MTYPE_NEWLIST,
-  MTYPE_NEWNODE,
   MTYPE_BGP,
-  MTYPE_PEER,
   MTYPE_PEER_CONF,
   MTYPE_PEER_GROUP,
   MTYPE_BGP_STATIC,
@@ -150,8 +143,6 @@ enum
   MTYPE_KEY,
   MTYPE_RIP,
   MTYPE_RIP_INTERFACE,
-  MTYPE_ROUTE,
-  MTYPE_ROUTE_ENTRY,
   MTYPE_RIP_DISTANCE,
   MTYPE_BGP_DISTANCE,
   MTYPE_OSPF_DISTANCE,
@@ -161,6 +152,11 @@ enum
   MTYPE_BGP_NEXTHOP_CACHE,
   MTYPE_BGP_DAMP_INFO,
   MTYPE_BGP_DAMP_ARRAY,
+  MTYPE_BGP_ANNOUNCE,
+  MTYPE_BGP_ATTR_QUEUE,
+  MTYPE_BGP_ROUTE_QUEUE,
+  MTYPE_OSPF_IF_INFO,
+  MTYPE_OSPF_IF_PARAMS,
   MTYPE_MAX
 };
 
