@@ -54,327 +54,8 @@ ospf6_neighbor_last_dbdesc_release (struct thread *thread)
   return 0;
 }
 
-
 
 
-/* lookup lsa on lsa list of neighbor for dbdesc retransmit. */
-struct ospf6_lsa *
-ospf6_neighbor_dbdesc_lsa_lookup (struct ospf6_lsa *lsa,
-                                  struct ospf6_neighbor *o6n)
-{
-  if (listnode_lookup (o6n->dbdesc_lsa, lsa))
-    {
-#ifndef NDEBUG
-      if (!listnode_lookup (lsa->dbdesc_neighbor, o6n))
-        assert (0);
-#endif /* NDEBUG */
-      return lsa;
-    }
-  return NULL;
-}
-
-/* add lsa to summary list of neighbor */
-void
-ospf6_neighbor_dbdesc_lsa_add (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  if (ospf6_neighbor_summary_lookup (lsa, o6n))
-    return;
-
-  listnode_add (o6n->dbdesc_lsa, lsa);
-  listnode_add (lsa->dbdesc_neighbor, o6n);
-  ospf6_lsa_lock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: locked to be send in dbdesc to neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove lsa from summary list of neighbor */
-void
-ospf6_neighbor_dbdesc_lsa_remove (struct ospf6_lsa *lsa,
-                                  struct ospf6_neighbor *o6n)
-{
-  if (! ospf6_neighbor_dbdesc_lsa_lookup (lsa, o6n))
-    return;
-
-  listnode_delete (o6n->dbdesc_lsa, lsa);
-  listnode_delete (lsa->dbdesc_neighbor, o6n);
-  ospf6_lsa_unlock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: unlocked from being send in dbdesc to neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove all lsa from summary list of neighbor */
-void
-ospf6_neighbor_dbdesc_lsa_remove_all (struct ospf6_neighbor *o6n)
-{
-  struct ospf6_lsa *lsa;
-  listnode n;
-  while (listcount (o6n->dbdesc_lsa))
-    {
-      n = listhead (o6n->dbdesc_lsa);
-      lsa = (struct ospf6_lsa *) getdata (n);
-      ospf6_neighbor_dbdesc_lsa_remove (lsa, o6n);
-    }
-  return;
-}
-
-/* lookup lsa on summary list of neighbor */
-struct ospf6_lsa *
-ospf6_neighbor_summary_lookup (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  if (listnode_lookup (o6n->summarylist, lsa))
-    {
-#ifndef NDEBUG
-      if (! listnode_lookup (lsa->summary_nbr, o6n))
-        assert (0);
-#endif /* NDEBUG */
-      return lsa;
-    }
-  return NULL;
-}
-
-/* add lsa to summary list of neighbor */
-void
-ospf6_neighbor_summary_add (struct ospf6_lsa *lsa,
-                            struct ospf6_neighbor *o6n)
-{
-  if (ospf6_neighbor_summary_lookup (lsa, o6n))
-    return;
-
-  listnode_add (o6n->summarylist, lsa);
-  listnode_add (lsa->summary_nbr, o6n);
-  ospf6_lsa_lock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: locked from summary-list in neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove lsa from summary list of neighbor */
-void
-ospf6_neighbor_summary_remove (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  if (! ospf6_neighbor_summary_lookup (lsa, o6n))
-    return;
-
-  listnode_delete (o6n->summarylist, lsa);
-  listnode_delete (lsa->summary_nbr, o6n);
-  ospf6_lsa_unlock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: unlocked from summary-list in neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove all lsa from summary list of neighbor */
-void
-ospf6_neighbor_summary_remove_all (struct ospf6_neighbor *o6n)
-{
-  struct ospf6_lsa *lsa;
-  listnode n;
-  while (listcount (o6n->summarylist))
-    {
-      n = listhead (o6n->summarylist);
-      lsa = (struct ospf6_lsa *) getdata (n);
-      ospf6_neighbor_summary_remove (lsa, o6n);
-    }
-  return;
-}
-
-/* lookup lsa on request list of neighbor */
-/* this lookup is different from others, because this lookup is to find
-   the same LSA instance of different memory space */
-struct ospf6_lsa *
-ospf6_neighbor_request_lookup (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  listnode n;
-  struct ospf6_lsa *p;
-
-  for (n = listhead (o6n->requestlist); n; nextnode (n))
-    {
-      p = (struct ospf6_lsa *) getdata (n);
-      if (ospf6_lsa_issame ((struct ospf6_lsa_header *)p->lsa_hdr,
-                            (struct ospf6_lsa_header *)lsa->lsa_hdr))
-        {
-#ifndef NDEBUG
-          if (! listnode_lookup (p->request_nbr, o6n))
-          assert (0);
-#endif /* NDEBUG */
-          return p;
-        }
-    }
-  return NULL;
-}
-
-/* add lsa to request list of neighbor */
-void
-ospf6_neighbor_request_add (struct ospf6_lsa *lsa,
-                            struct ospf6_neighbor *o6n)
-{
-  if (ospf6_neighbor_request_lookup (lsa, o6n))
-    return;
-
-  listnode_add (o6n->requestlist, lsa);
-  listnode_add (lsa->request_nbr, o6n);
-  ospf6_lsa_lock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: locked from request-list in neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove lsa from request list of neighbor */
-void
-ospf6_neighbor_request_remove (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  if (! ospf6_neighbor_request_lookup (lsa, o6n))
-    return;
-
-  listnode_delete (o6n->requestlist, lsa);
-  listnode_delete (lsa->request_nbr, o6n);
-  ospf6_lsa_unlock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: unlocked from request-list in neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove all lsa from request list of neighbor */
-void
-ospf6_neighbor_request_remove_all (struct ospf6_neighbor *o6n)
-{
-  listnode n;
-  struct ospf6_lsa *lsa;
-  while (listcount (o6n->requestlist))
-    {
-      n = listhead (o6n->requestlist);
-      lsa = (struct ospf6_lsa *) getdata (n);
-      ospf6_neighbor_request_remove (lsa, o6n);
-    }
-  return;
-}
-
-/* lookup lsa on retrans list of neighbor */
-struct ospf6_lsa *
-ospf6_neighbor_retrans_lookup (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  if (listnode_lookup (o6n->retranslist, lsa))
-    {
-#ifndef NDEBUG
-      if (! listnode_lookup (lsa->retrans_nbr, o6n))
-        assert (0);
-#endif /* NDEBUG */
-      return lsa;
-    }
-  return NULL;
-}
-
-/* add lsa to retrans list of neighbor */
-void
-ospf6_neighbor_retrans_add (struct ospf6_lsa *lsa,
-                            struct ospf6_neighbor *o6n)
-{
-  if (ospf6_neighbor_retrans_lookup (lsa, o6n))
-    return;
-
-  listnode_add (o6n->retranslist, lsa);
-  listnode_add (lsa->retrans_nbr, o6n);
-  ospf6_lsa_lock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: locked from retrans-list in neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  return;
-}
-
-/* remove lsa from retrans list of neighbor */
-void
-ospf6_neighbor_retrans_remove (struct ospf6_lsa *lsa,
-                               struct ospf6_neighbor *o6n)
-{
-  /* if not on retranslist, return */
-  if (! ospf6_neighbor_retrans_lookup (lsa, o6n))
-    return;
-
-  /* remove from retrans list */
-  listnode_delete (o6n->retranslist, lsa);
-  listnode_delete (lsa->retrans_nbr, o6n);
-  ospf6_lsa_unlock (lsa);
-
-#if 0
-  if (IS_OSPF6_DUMP_LSA)
-    zlog_info ("lsa: unlocked from retrans-list in neighbor %s: %s (lock:%d)",
-               o6n->str, lsa->str, lsa->lock);
-#endif
-
-  /* if this LSA is MaxAge, try to delete */
-  if (ospf6_lsa_is_maxage (lsa))
-    {
-      struct ospf6_lsa_header *lsa_header =
-        (struct ospf6_lsa_header *) lsa->lsa_hdr;
-
-      if (OSPF6_LSA_IS_SCOPE_LINKLOCAL (ntohs (lsa_header->type)))
-        ospf6_lsdb_check_maxage_linklocal (((struct ospf6_interface *)
-                                            lsa->scope)->interface->name);
-      else if (OSPF6_LSA_IS_SCOPE_AREA (ntohs (lsa_header->type)))
-        ospf6_lsdb_check_maxage_area (((struct ospf6_area *)
-                                       lsa->scope)->area_id);
-      else if (OSPF6_LSA_IS_SCOPE_AS (ntohs (lsa_header->type)))
-        ospf6_lsdb_check_maxage_as ();
-    }
-}
-
-/* remove all lsa from retrans list of neighbor */
-void
-ospf6_neighbor_retrans_remove_all (struct ospf6_neighbor *o6n)
-{
-  listnode n;
-  struct ospf6_lsa *lsa;
-  while (listcount (o6n->retranslist))
-    {
-      n = listhead (o6n->retranslist);
-      lsa = (struct ospf6_lsa *) getdata (n);
-      ospf6_neighbor_retrans_remove (lsa, o6n);
-    }
-  return;
-}
-
 void
 ospf6_neighbor_thread_cancel_all (struct ospf6_neighbor *o6n)
 {
@@ -386,23 +67,197 @@ ospf6_neighbor_thread_cancel_all (struct ospf6_neighbor *o6n)
     thread_cancel (o6n->send_update);
   o6n->send_update = (struct thread *) NULL;
 
-  if (o6n->thread_dbdesc)
-    thread_cancel (o6n->thread_dbdesc);
-  o6n->thread_dbdesc = (struct thread *) NULL;
+  if (o6n->thread_send_dbdesc)
+    thread_cancel (o6n->thread_send_dbdesc);
+  o6n->thread_send_dbdesc = (struct thread *) NULL;
+  if (o6n->thread_rxmt_dbdesc)
+    thread_cancel (o6n->thread_rxmt_dbdesc);
+  o6n->thread_rxmt_dbdesc = (struct thread *) NULL;
 
   if (o6n->thread_rxmt_lsreq)
     thread_cancel (o6n->thread_rxmt_lsreq);
   o6n->thread_rxmt_lsreq = (struct thread *) NULL;
 }
 
+void
+ospf6_neighbor_lslist_clear (struct ospf6_neighbor *nei)
+{
+  ospf6_lsdb_remove_all (nei->summary_list);
+  ospf6_lsdb_remove_all (nei->request_list);
+  ospf6_lsdb_remove_all (nei->retrans_list);
+  ospf6_lsdb_remove_all (nei->dbdesc_list);
+}
 
 void
-ospf6_neighbor_list_remove_all (struct ospf6_neighbor *o6n)
+ospf6_neighbor_summary_add (struct ospf6_lsa *lsa,
+                            struct ospf6_neighbor *nei)
 {
-  ospf6_neighbor_dbdesc_lsa_remove_all (o6n);
-  ospf6_neighbor_summary_remove_all (o6n);
-  ospf6_neighbor_request_remove_all (o6n);
-  ospf6_neighbor_retrans_remove_all (o6n);
+  struct ospf6_lsa *summary;
+
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor %s summary-list: add %s", nei->str, lsa->str);
+
+  ospf6_lsa_age_current (lsa);
+  summary = ospf6_lsa_summary_create (lsa->header);
+  ospf6_lsdb_add (summary, nei->summary_list);
+}
+
+void
+ospf6_neighbor_summary_remove (struct ospf6_lsa *lsa,
+                               struct ospf6_neighbor *nei)
+{
+  struct ospf6_lsa *summary;
+
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor %s summary-list: remove %s",
+               nei->str, lsa->str);
+
+  summary = ospf6_lsdb_lookup_lsdb (lsa->header->type, lsa->header->id,
+                                    lsa->header->adv_router, nei->summary_list);
+  if (! summary)
+    {
+      if (IS_OSPF6_DUMP_NEIGHBOR)
+        zlog_info ("Can't remove %s's summary-list: no such instance: %s",
+                   nei->str, lsa->str);
+      return;
+    }
+
+  ospf6_lsdb_remove (summary, nei->summary_list);
+}
+
+void
+ospf6_neighbor_request_add (struct ospf6_lsa *lsa,
+                            struct ospf6_neighbor *nei)
+{
+  struct ospf6_lsa *summary;
+
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor %s request-list: add %s", nei->str, lsa->str);
+
+  ospf6_lsa_age_current (lsa);
+  summary = ospf6_lsa_summary_create (lsa->header);
+  ospf6_lsdb_add (summary, nei->request_list);
+}
+
+void
+ospf6_neighbor_request_remove (struct ospf6_lsa *lsa,
+                               struct ospf6_neighbor *nei)
+{
+  struct ospf6_lsa *summary;
+
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor %s request-list: remove %s",
+               nei->str, lsa->str);
+
+  summary = ospf6_lsdb_lookup_lsdb (lsa->header->type, lsa->header->id,
+                                    lsa->header->adv_router, nei->request_list);
+  if (! summary)
+    {
+      if (IS_OSPF6_DUMP_NEIGHBOR)
+        zlog_info ("Can't remove %s's request-list: no such instance: %s",
+                   nei->str, lsa->str);
+      return;
+    }
+
+  ospf6_lsdb_remove (summary, nei->request_list);
+}
+
+void
+ospf6_neighbor_retrans_add (struct ospf6_lsa *lsa,
+                            struct ospf6_neighbor *nei)
+{
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor retrans-list: add %s", lsa->str);
+
+  ospf6_lsdb_add (lsa, nei->retrans_list);
+}
+
+void
+ospf6_neighbor_retrans_remove (struct ospf6_lsa *lsa,
+                               struct ospf6_neighbor *nei)
+{
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor retrans-list: remove %s", lsa->str);
+
+  ospf6_lsdb_remove (lsa, nei->retrans_list);
+}
+
+void
+ospf6_neighbor_dbdesc_add (struct ospf6_lsa *lsa,
+                           struct ospf6_neighbor *nei)
+{
+  struct ospf6_lsa *summary;
+
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor %s dbdesc-list: add %s", nei->str, lsa->str);
+
+  ospf6_lsa_age_current (lsa);
+  summary = ospf6_lsa_summary_create (lsa->header);
+  ospf6_lsdb_add (summary, nei->dbdesc_list);
+}
+
+void
+ospf6_neighbor_dbdesc_remove (struct ospf6_lsa *lsa,
+                              struct ospf6_neighbor *nei)
+{
+  struct ospf6_lsa *summary;
+
+  if (IS_OSPF6_DUMP_NEIGHBOR)
+    zlog_info ("Neighbor %s dbdesc-list: remove %s",
+               nei->str, lsa->str);
+
+  summary = ospf6_lsdb_lookup_lsdb (lsa->header->type, lsa->header->id,
+                                    lsa->header->adv_router, nei->dbdesc_list);
+  if (! summary)
+    {
+      if (IS_OSPF6_DUMP_NEIGHBOR)
+        zlog_info ("Can't remove %s's dbdesc-list: no such instance: %s",
+                   nei->str, lsa->str);
+      return;
+    }
+
+  ospf6_lsdb_remove (summary, nei->dbdesc_list);
+}
+
+
+/* prepare summary-list of his neighbor structure */
+void
+ospf6_neighbor_dbex_init (struct ospf6_neighbor *nei)
+{
+  struct ospf6_lsdb_node *node;
+
+  /* clear summary-list */
+  ospf6_lsdb_remove_all (nei->summary_list);
+
+  /* AS scope LSAs */
+  for (node = ospf6_lsdb_head (nei->ospf6_interface->area->ospf6->lsdb);
+       node; node = ospf6_lsdb_next (node))
+    {
+      if (ospf6_lsa_is_maxage (node->lsa))
+        ospf6_neighbor_retrans_add (node->lsa, nei);
+      else
+        ospf6_neighbor_summary_add (node->lsa, nei);
+    }
+
+  /* AREA scope LSAs */
+  for (node = ospf6_lsdb_head (nei->ospf6_interface->area->lsdb);
+       node; node = ospf6_lsdb_next (node))
+    {
+      if (ospf6_lsa_is_maxage (node->lsa))
+        ospf6_neighbor_retrans_add (node->lsa, nei);
+      else
+        ospf6_neighbor_summary_add (node->lsa, nei);
+    }
+
+  /* INTERFACE scope LSAs */
+  for (node = ospf6_lsdb_head (nei->ospf6_interface->lsdb);
+       node; node = ospf6_lsdb_next (node))
+    {
+      if (ospf6_lsa_is_maxage (node->lsa))
+        ospf6_neighbor_retrans_add (node->lsa, nei);
+      else
+        ospf6_neighbor_summary_add (node->lsa, nei);
+    }
 }
 
 /* create ospf6_neighbor */
@@ -427,11 +282,12 @@ ospf6_neighbor_create (u_int32_t router_id, struct ospf6_interface *o6i)
   new->router_id = router_id;
   inet_ntop (AF_INET, &router_id, buf, sizeof (buf));
   snprintf (new->str, sizeof (new->str), "%s%%%s", buf, o6i->interface->name);
-  new->inactivity_timer = (struct thread *)NULL;
-  new->summarylist = list_new ();
-  new->retranslist = list_new ();
-  new->requestlist = list_new ();
-  new->dbdesc_lsa = list_new ();
+  new->inactivity_timer = (struct thread *) NULL;
+
+  new->summary_list = ospf6_lsdb_create ();
+  new->request_list = ospf6_lsdb_create ();
+  new->retrans_list = ospf6_lsdb_create ();
+  new->dbdesc_list = ospf6_lsdb_create ();
 
   listnode_add (o6i->neighbor_list, new);
   new->ospf6_interface = o6i;
@@ -443,12 +299,14 @@ void
 ospf6_neighbor_delete (struct ospf6_neighbor *o6n)
 {
   ospf6_neighbor_thread_cancel_all (o6n);
-  ospf6_neighbor_list_remove_all (o6n);
+  ospf6_neighbor_lslist_clear (o6n);
 
   list_free (o6n->dbdesc_lsa);
-  list_free (o6n->summarylist);
-  list_free (o6n->requestlist);
-  list_free (o6n->retranslist);
+
+  ospf6_lsdb_delete (o6n->summary_list);
+  ospf6_lsdb_delete (o6n->request_list);
+  ospf6_lsdb_delete (o6n->retrans_list);
+  ospf6_lsdb_delete (o6n->dbdesc_list);
 
   XFREE (MTYPE_OSPF6_NEIGHBOR, o6n);
 }
@@ -526,12 +384,12 @@ ospf6_neighbor_show_detail (struct vty *vty, struct ospf6_neighbor *o6n)
            (DD_IS_MBIT_SET (o6n->dbdesc_bits) ? "M" : "-"),
            (DD_IS_MSBIT_SET (o6n->dbdesc_bits) ? "m" : "s"),
            VTY_NEWLINE);
-  vty_out (vty, "    His Ifindex of myside: %lu%s",
+  vty_out (vty, "    His Ifindex of myside: %d%s",
                 o6n->ifid, VTY_NEWLINE);
   vty_out (vty, "    His DRDecision: DR %s, BDR %s%s",
                 hisdr, hisbdr, VTY_NEWLINE);
   vty_out (vty, "    Last received DbDesc: opt:%s"
-                " ifmtu:%hu bit:%s%s%s seqnum:%lu%s",
+                " ifmtu:%hu bit:%s%s%s seqnum:%d%s",
                 "xxx", ntohs (o6n->last_dd.ifmtu),
                 (DD_IS_IBIT_SET (o6n->last_dd.bits) ? "I" : "-"),
                 (DD_IS_MBIT_SET (o6n->last_dd.bits) ? "M" : "-"),
@@ -539,12 +397,14 @@ ospf6_neighbor_show_detail (struct vty *vty, struct ospf6_neighbor *o6n)
                 ntohl (o6n->last_dd.seqnum), VTY_NEWLINE);
   vty_out (vty, "    Number of LSAs in DbDesc retransmitting: %d%s",
                 listcount (o6n->dbdesc_lsa), VTY_NEWLINE);
+#if 0
   vty_out (vty, "    Number of LSAs in SummaryList: %d%s",
                 listcount (o6n->summarylist), VTY_NEWLINE);
   vty_out (vty, "    Number of LSAs in RequestList: %d%s",
                 listcount (o6n->requestlist), VTY_NEWLINE);
   vty_out (vty, "    Number of LSAs in RetransList: %d%s",
                 listcount (o6n->retranslist), VTY_NEWLINE);
+#endif
   vty_out (vty, "    %-16s %5d times, %-16s %5d times%s",
                 "SeqnumMismatch", o6n->ospf6_stat_seqnum_mismatch,
                 "BadLSReq", o6n->ospf6_stat_bad_lsreq, VTY_NEWLINE);
@@ -573,7 +433,7 @@ ospf6_neighbor_timestamp_hello (struct ospf6_neighbor *o6n)
   if (o6n->tv_last_hello_received.tv_sec)
     {
       ospf6_timeval_sub (&now, &o6n->tv_last_hello_received, &interval);
-      zlog_info ("Hello Interval %s : %d msec",
+      zlog_info ("Hello Interval %s : %ld msec",
                   o6n->str, interval.tv_sec * 1000 + interval.tv_usec % 1000);
     }
   o6n->tv_last_hello_received.tv_sec = now.tv_sec;
@@ -582,13 +442,13 @@ ospf6_neighbor_timestamp_hello (struct ospf6_neighbor *o6n)
 
 DEFUN (show_ipv6_ospf6_neighbor_ifname_nbrid_detail,
        show_ipv6_ospf6_neighbor_ifname_nbrid_detail_cmd,
-       "show ipv6 ospf6 neighbor IFNAME NBR_ID detail",
+       "show ipv6 ospf6 neighbor IFNAME A.B.C.D detail",
        SHOW_STR
        IP6_STR
        OSPF6_STR
        "Neighbor list\n"
        IFNAME_STR
-       "A.B.C.D OSPF6 neighbor Router ID in IP address format\n"
+       "OSPF6 neighbor Router ID in IP address format\n"
        "detailed infomation\n"
        )
 {
@@ -599,8 +459,9 @@ DEFUN (show_ipv6_ospf6_neighbor_ifname_nbrid_detail,
   struct ospf6_area *area;
   listnode i, j, k;
 
-  i = j = k = NULL;
+  OSPF6_CMD_CHECK_RUNNING ();
 
+  i = j = k = NULL;
   vty_out (vty, "%-15s %-3s %-8s %-15s %-15s %s[%s]%s",
      "RouterID", "Pri", "State", "DR", "BDR", "I/F", "State", VTY_NEWLINE);
 
@@ -608,18 +469,29 @@ DEFUN (show_ipv6_ospf6_neighbor_ifname_nbrid_detail,
     {
       ifp = if_lookup_by_name (argv[0]);
       if (!ifp)
-        return CMD_ERR_NO_MATCH;
+        {
+          vty_out (vty, "no such interface: %s", argv[0]);
+          return CMD_SUCCESS;
+        }
 
       ospf6_interface = (struct ospf6_interface *) ifp->info;
       if (!ospf6_interface)
-        return CMD_ERR_NO_MATCH;
+        {
+          vty_out (vty, "ospf not enabled on interface: %s", argv[0]);
+          return CMD_SUCCESS;
+        }
 
       if (argc > 1)
         {
           inet_pton (AF_INET, argv[1], &router_id);
           nbr = ospf6_neighbor_lookup (router_id, ospf6_interface);
           if (!nbr)
-            return CMD_ERR_NO_MATCH;
+            {
+              vty_out (vty, "neighbor %s not found on %s", argv[1],
+                       ospf6_interface->interface->name);
+              return CMD_SUCCESS;
+            }
+
           if (argc == 3)
             ospf6_neighbor_show_detail (vty, nbr);
           else
@@ -683,7 +555,7 @@ ALIAS (show_ipv6_ospf6_neighbor_ifname_nbrid_detail,
 
 DEFUN (show_ipv6_ospf6_neighborlist,
        show_ipv6_ospf6_neighborlist_cmd,
-       "show ipv6 ospf6 (summary-list|request-list|retransmission-list)",
+       "show ipv6 ospf6 (summary-list|request-list|retrans-list)",
        SHOW_STR
        IP6_STR
        OSPF6_STR
@@ -697,7 +569,11 @@ DEFUN (show_ipv6_ospf6_neighborlist,
   struct ospf6_neighbor *o6n;
   listnode i, j, k, l;
   struct ospf6_lsa *lsa;
-  list lslist = NULL;
+  struct ospf6_lsdb *lsdb = NULL;
+  char adv_router[128];
+  struct ospf6_lsdb_node *node;
+
+  OSPF6_CMD_CHECK_RUNNING ();
 
   i = j = k = l = NULL;
 
@@ -712,18 +588,29 @@ DEFUN (show_ipv6_ospf6_neighborlist,
               o6n = (struct ospf6_neighbor *) getdata (k);
 
               if (strncmp (argv[0], "sum", 3) == 0)
-                lslist = o6n->summarylist;
+                lsdb = o6n->summary_list;
               else if (strncmp (argv[0], "req", 3) == 0)
-                lslist = o6n->requestlist;
+                lsdb = o6n->request_list;
               else if (strncmp (argv[0], "ret", 3) == 0)
-                lslist = o6n->retranslist;
+                lsdb = o6n->retrans_list;
 
               vty_out (vty, "neighbor %s on interface %s: %d%s", o6n->str,
-                       o6i->interface->name, listcount (lslist), VTY_NEWLINE);
-              for (l = listhead (lslist); l; nextnode (l))
+                       o6i->interface->name, lsdb->count,
+                       VTY_NEWLINE);
+              for (node = ospf6_lsdb_head (lsdb); node;
+                   node = ospf6_lsdb_next (node))
                 {
-                  lsa = (struct ospf6_lsa *) getdata (l);
-                  vty_out (vty, "  %s%s", lsa->str, VTY_NEWLINE);
+                  lsa = node->lsa;
+
+                  inet_ntop (AF_INET, &lsa->header->adv_router, adv_router,
+                             sizeof (adv_router));
+                  vty_out (vty, "  %s-LSA ID: %d Adv_router: %s%s",
+                           ospf6_lsa_type_string (lsa->header->type),
+                           ntohl (lsa->header->id), adv_router, VTY_NEWLINE);
+                  vty_out (vty, "    Age: %hu SeqNum: %d Cksum: %hx Len: %hu%s",
+                           ntohs (lsa->header->age), ntohl (lsa->header->seqnum),
+                           ntohs (lsa->header->checksum), ntohs (lsa->header->length),
+                           VTY_NEWLINE);
                 }
             }
         }
